#pragma IndependentModule = DemoSetupTools
#pragma version = 1.00

// updater headers
static constant kProjectID=21825
static strconstant ksShortTitle="DemoSetupTools"

// Demo Setup Tools
// 2022-02-12 jjw
// hide all Igor menus
// protect the graph and panel
// show a demo menu
// send results to notebook
// send results to layout
// requires Igor Pro 8 or greater

// US letter page in landscape with 1/4 in margin
Static Constant spgwidth = 792
Static Constant spgheight = 612
Static Constant spgmargin = 18
// graph takes this fraction of layout page
Static Constant sgfrac = 0.75

// *** Make Demo ***

Static Function AfterCompiledHook()
	
	NVAR/Z/SDFR=root: demomode
	
	ShowIgorMenus
	if (!NVAR_exists(demomode))
		initialize_Demo(0)
	else
		initialize_Demo(demomode)
	endif
	
	return 0
end

// initialize

Static Function initialize_Demo(variable how)
	
	NVAR/Z/SDFR=root: UIControl, pgwidth, pgheight, pgmargin, gfrac

	if (!DataFolderExists("root:Packages:DemoSetupTools"))
		NewDataFolder/O/S root:Packages
		NewDataFolder/O/S DemoSetupTools
		variable/G guic, gpgwidth, gpgheight, gpgmargin, ggfrac
		string/G ggname, gpname
	else
		SetDataFolder root:Packages:DemoSetupTools
	endif
	
	DFREF pf = root:Packages:DemoSetupTools
	NVAR/SDFR=pf guic, gpgwidth, gpgheight, gpgmargin, ggfrac
	SVAR/SDFR=pf ggname, gpname
	
	ggname = ""
	gpname = ""
	
	if (NVAR_exists(UIControl))
		guic = UIControl
	else
		guic = 4095
	endif
		
	if (NVAR_exists(pgwidth))
		gpgwidth = pgwidth
	else
		gpgwidth = spgwidth
	endif
		
	if (NVAR_exists(pgheight))
		gpgheight = pgheight
	else
		gpgheight = spgheight
	endif
		
	if (NVAR_exists(pgmargin))
		gpgmargin = pgmargin
	else
		gpgmargin = spgmargin
	endif
		
	if (NVAR_exists(gfrac))
		ggfrac = gfrac
	else
		ggfrac = sgfrac
	endif
	
	ggname = WinName(0,1,1)
	gpname = WinName(0,64,1)
	
	SetDataFolder root:
	
	if (how==1)
		HideIgorMenus
		if (strlen(ggname)!=0)
			ModifyGraph/W=$ggname UIControl=guic
		endif
		if (strlen(gpname)!=0)
			ModifyPanel/W=$gpname noEdit=1, fixedSize=1
		endif
	else
		ShowIgorMenus
		if (strlen(ggname)!=0)
			ModifyGraph/W=$ggname UIControl=0
		endif
		if (strlen(gpname)!=0)
			ModifyPanel/W=$gpname noEdit=0, fixedSize=0
		endif
	endif
	
	return 0
end

// *** Layout ***

// create layout
Function layout_FrontSet([string notestr])
	
	DFREF pf = root:Packages:DemoSetupTools
	SVAR/SDFR=pf ggname, gpname
	
	if (ParamIsDefault(notestr))
		layout_results(ggname, gpname)
	else
		layout_results(ggname, gpname, notestr = notestr)
	endif
	
	return 0
end

// set gfill = 1 to have graph fill its region
Static Function layout_results(string graphname, string panelname, [string notestr, variable gfill])

	DFREF pf = root:Packages:DemoSetupTools
	NVAR/SDFR=pf gpgwidth, gpgheight, gpgmargin, ggfrac
	
	variable gwidth, gheight, pwidth, pheight
	variable grwidth, prwidth
	variable gratio, pratio, wthratio
	variable toffset, tsize
	variable gonly = 0
	string dpstr
	
	// graph & panel or just graph
	
	if (strlen(panelname)!=0)
		
		// sizes of regions for appending
		
		grwidth = ggfrac*(gpgwidth-2*gpgmargin)
		prwidth = (1 - ggfrac)*(gpgwidth-3*gpgmargin)
		
		gratio = grwidth/(gpgheight-2*gpgmargin)
		pratio = prwidth/(gpgheight-2*gpgmargin-4*gpgmargin)
		
		// validity checks	 and window sizes
		
		DoWindow/F $graphname
		if (v_flag == 0)
			return -1
		endif	
		GetWindow/Z $graphname wsize
		gwidth = V_right - V_left; gheight = V_bottom - V_top
		//print gwidth, gheight
		
		DoWindow/F $panelname
		if (v_flag == 0)
			return -1
		endif
		SavePICT/SNAP=1/O/WIN=$panelname/P=_PictGallery_/E=-5 as "PanelSnapShot"
		GetWindow/Z $panelname wsize
		pwidth = V_right - V_left; pheight = V_bottom - V_top
	else
		
		gonly = 1
		
		// sizes of regions for appending
		
		grwidth = (gpgwidth-2*gpgmargin)
		prwidth = 0
		
		gratio = grwidth/(gpgheight-2*gpgmargin)
		pratio = 0
		
		// validity checks	 and window sizes
		
		DoWindow/F $graphname
		if (v_flag == 0)
			return -1
		endif	
		GetWindow/Z $graphname wsize
		gwidth = V_right - V_left; gheight = V_bottom - V_top
	endif
		
	// strings for stamp
	
	string enstr, strdt, strsys, strprbx
	
	enstr = IgorInfo(1) + "\r"
	strdt = date() + " " + time()
	strsys = IgorInfo(7) + " : " + IgorInfo(2)	
	strprbx = "\\Z10\\JR" + enstr + strsys
	strprbx += "\r" + strdt
	if (!ParamIsDefault(notestr))
		strprbx += "\r" + notestr
	endif
	
	// remove window to refresh
	
	KillWindow/Z SnapshotResultsLayout
	
	NewLayout/K=1/W=(30,50,600,500) as "Results Layout"
	DoWindow/C SnapshotResultsLayout
	LayoutPageAction size=(gpgwidth,gpgheight),margins=(gpgmargin,gpgmargin,gpgmargin,gpgmargin)
	ModifyLayout mag=0.5
	
	// append graph
	
	if (ParamIsDefault(gfill))
		wthratio = gwidth/gheight
		if (wthratio > gratio)
			// too wide, shrink height by ratio
			tsize = gheight*(grwidth/gwidth)
			toffset = 2*gpgmargin
			AppendLayoutObject/F=0/T=0/R=(gpgmargin,toffset,grwidth+gpgmargin,tsize+toffset) Graph $graphname
			//print "too wide", wthratio, grwidth/tsize
		else
			// too tall, shrink width by ratio
			toffset = gpgmargin
			tsize = gwidth*(gpgheight-2*gpgmargin)/gheight
			AppendLayoutObject/F=0/T=0/R=(gpgmargin,toffset,tsize+gpgmargin,gpgheight-gpgmargin) Graph $graphname
			//print "too tall", wthratio, tsize/(pgheight-2*pgmargin)
		endif
	else
		AppendLayoutObject/F=0/T=0/R=(gpgmargin,gpgmargin,grwidth+gpgmargin,gpgheight-2*gpgmargin) Graph $graphname
	endif
	
	// append panel and text stamp
	
	SetDrawLayer ProgBack
	if (!gonly)
		wthratio = pwidth/pheight
		if (wthratio > pratio)
			//print "wide panel", pwidth, pheight, wthratio, pratio
			toffset = pratio*pheight - 2*gpgmargin
			pratio = prwidth/pwidth
		else
			//print "tall panel", pwidth, pheight
			toffset = gpgmargin
			pratio = (gpgheight-2*gpgmargin-4*gpgmargin)/pheight
		endif		
		sprintf dpstr, "DrawPICT %f,%f,%f,%f,PanelSnapShot",grwidth+gpgmargin,toffset,pratio,pratio
		Execute/Q/Z dpstr
		//DrawPICT grwidth+pgmargin,toffset,pratio,pratio,PanelSnapShot	
		DrawText grwidth+gpgmargin, gpgheight-1.5*gpgmargin, strprbx
	else
		DrawText 1.5*gpgmargin, gpgheight-1.5*gpgmargin, strprbx
	endif
	SetDrawLayer UserFront
	return 0
end

// *** Notebook ***

// create notebook
Function notebook_FrontSet([string notestr])
	
	DFREF pf = root:Packages:DemoSetupTools
	SVAR/SDFR=pf ggname, gpname
	
	if (ParamIsDefault(notestr))
		notebook_results(ggname, gpname)
	else
		notebook_results(ggname, gpname, notestr = notestr)
	endif
	
	return 0
end

Static Function notebook_results(string graphname, string panelname, [string notestr])

	string ntbkstr, ntbkName
	
	ntbkstr = IgorInfo(1) + "_ntbk" 
	
	ntbkName = CleanupName(ntbkstr,0)
	
	if (strlen(WinList(ntbkName,";","WIN:16")) == 0)
		NewNotebook/F=1/N=$ntbkName as ntbkstr
		create_NtbkRulers(ntbkName)
		put_FirstHeaders(ntbkName)
	endif
	
	// put information

	if (ParamIsDefault(notestr))
		put_Info(ntbkName)
	else
		put_Info(ntbkName, notestr = notestr)
	endif
	
	// put the graph and panel

	put_theGraph(ntbkName, graphname)
	if (strlen(panelname)!=0)
		put_thePanel(ntbkName, panelname)
	endif
	
	return 0
end

Static Function put_theGraph(string theNotebook, string thegraph)

	string gtitle, gname
	variable wsize, gwidth
	
	GetWindow/Z $thegraph, wtitle
	gtitle = thegraph + " - " +  s_value
	gname = "\rGraph: " + gtitle + "\r"
	
	Notebook $theNotebook, scaling={50,50}, picture={$thegraph,-5,1}
	Notebook $theNotebook, text = gname
	return 0
end

Static Function put_thePanel(string theNotebook, string thepanel)

	string gtitle, gname
	variable wsize, gwidth
	
	GetWindow/Z $thepanel, wtitle
	gtitle = thepanel + " - " +  s_value
	gname = "\rPanel: " + gtitle + "\r"
	SavePICT/SNAP=1/O/WIN=$thepanel/P=_PictGallery_/E=-5 as thepanel
	
	Notebook $theNotebook, scaling={50,50}, picture={$thepanel,-5,1}
	Notebook $theNotebook, text = gname
	return 0
end

Static Function put_Info(string theNotebook, [string notestr])

	string tstr
	
	Notebook $theNotebook ruler=Subheading
	Notebook $theNotebook text="## Information\r"
	Notebook $theNotebook ruler=Normal
	tstr = "#date " + date() + " #time " + time() + "\r"
	if (!ParamIsDefault(notestr))
		tstr += "#note " + notestr + "\r"
	endif
	Notebook $theNotebook text = tstr	
	return 0
end

Static Function Create_NtbkRulers(string theNtbk)
	
	Notebook $theNtbk newRuler=Heading, spacing={12,6,0}, rulerDefaults={"",14,1,(0,0,0)}
	Notebook $theNtbk newRuler=Subheading, spacing={6,6,0}, rulerDefaults={"",12,2,(0,0,0)}
	Notebook $theNtbk Ruler=Normal, spacing={0,0,0}, rulerDefaults={"",12,0,(0,0,0)}
	return 0
end

Static Function put_FirstHeaders(string theNotebook)

	string tstr
	
	Notebook $theNotebook ruler=Heading
	tstr = "# Start of Notebook\r"
	Notebook $theNotebook text = tstr
	
	Notebook $theNotebook ruler=Normal
	tstr = "#date " + date() + "\r"
	Notebook $theNotebook text = tstr
	
	Notebook $theNotebook ruler=Heading
	tstr = "# Experiment Information\r"
	Notebook $theNotebook text = tstr
		
	Notebook $theNotebook ruler=Normal
	tstr = "Project Name: " + IgorInfo(1)
	tstr += "\rIgor Version Information: " + IgorInfo(3)
	tstr += "\rUser Name: " + IgorInfo(7) + "\r"
	Notebook $theNotebook text = tstr

	return 0
end